# -----------------------------------------------------------------------------
# Rolling demo
#
# Copyright 2005 Ron Dippold
# -----------------------------------------------------------------------------

import pygame
from pygame.locals import *

from Plane import Fighter, Plane
from TextSprite import *
from Boom import Boom
from ImageSet import ImageSet
import Util
import TimedEvent
import Game


( STATE_1_1, STATE_1_2, STATE_1_3, STATE_2_1 ) = range(4)

class demo:
	sprite_set = None 	# Our demo sprite set
	PLAY_RECT = None	# The area we can play in
	
	clouds = [None, None] # fluffy clouds drawn by planes
	planes = [None, None] # The planes

	state  = None		# current demo state


# -----------------------------------------------------------------------------
# Demo mode rolling staaaaaart!
# -----------------------------------------------------------------------------

def start( _sprite_set, _rect ):
	"""
	Demo start - just pass in the area (_rect) we can play around int
	"""
	# Save the passed in globals (tsk), empty sprite list just in case
	demo.PLAY_RECT = _rect
	demo.sprite_set = _sprite_set
	demo.sprite_set.empty()
	
	# Clear all outstanding events, though there shouldn't be any
	TimedEvent.remove_all()

	demo_part_1_1()

def stop():
	"""
	Stop the demo mode, clean up any crud
	"""
	TimedEvent.remove_all()
	demo.sprite_set.empty()

def pump( _seconds ):

	# We're just waiting for the planes to draw their clouds, then collide
	if demo.state==STATE_1_1:
		if demo.planes[0].done and demo.planes[1].done:
			if demo.planes[0].rect.colliderect( demo.planes[1].rect ):
				demo_part_1_2()
		

# -----------------------------------------------------------------------------
# 
# part_1_1	- Planes draw clouds in sky, collide
# part_1_2	- Explosions where planes collided, revealing text
#
# -----------------------------------------------------------------------------

def demo_part_1_1( _seconds=None, _save=None ):
	demo.state = STATE_1_1

	# Just in case
	Game.level_clear()
	demo.sprite_set.empty()

	# Verison # at the upper right
	text = TextSprite( "Version "+Game.VERSION, 18, (128,128,128), (256, 20) )
	text.rect.right = demo.PLAY_RECT.right - 20
	demo.sprite_set.add( text )

	# 'Pygame (and SDL) present' - left edge
	text = TextSprite( "Pygame Classix presents:", 24, (255,255,0), (256, 20) )
	text.rect.left = 20
	demo.sprite_set.add( text )

	y = text.rect.bottom+20

	# This must be a multiple of the smoke_l/smoke_s size
	cloud_step = DemoCloud.preload()*2

	# Add the 'Wavy' cloud/plane set
	demo.clouds[0] = DemoCloud( (20,y), (16, 5), cloud_step )
	demo.sprite_set.add( demo.clouds[0] )
	demo.planes[0] = DemoFighter( (0,0), demo.clouds[0], wavy_moves )
	demo.sprite_set.add( demo.planes[0] )

	# Add the 'Navy' cloud/plane set
	demo.clouds[1] = DemoCloud( (276,y), (16, 5), cloud_step )
	demo.sprite_set.add( demo.clouds[1] )
	demo.planes[1] = DemoFighter( (1,1), demo.clouds[1], navy_moves )
	demo.sprite_set.add( demo.planes[1] )


def demo_part_1_2( _seconds=None, _save=None ):
	demo.state = STATE_1_2
	
	# save the positions for placing text
	y = demo.planes[0].rect.centery
	x = demo.PLAY_RECT.centerx

	# Kill off the planes
	for plane in demo.planes:
		plane.kill()

	# Add some more commentary
	text = TextSprite( "Created using Python and Pygame", \
						   24, (255,255,0), (x,y) )
	demo.sprite_set.add( text )
	boomy = text.rect.bottom-3
	y = text.rect.bottom+10
	width = text.rect.width

	text = TextSprite( "Copyright 2005 Ron Dippold",
					   24, (255,0,255), (x,y) )
	demo.sprite_set.add( text )
	y = text.rect.bottom+10
	width = max( width, text.rect.width )

	# Add explosions covering up the comments
	boom_width = Boom.max_width()

	# Pad it out to an even number of widths
	count = int((width+boom_width-1)/boom_width)+2
	width = count * boom_width
	bx = x-(width/2)

	# Fill the area with explosions lasting 1 second
	demo.booms = [ Boom( Boom.SIZE_LARGE, (bx+boom_width*i, boomy), (0,0), 1 ) \
				   for i in range(count) ]
	for boom in demo.booms:
		demo.sprite_set.add( boom )

	# Move on to next part of demo
	TimedEvent.add( 2, demo_part_1_3, _data=(x,y) )

def demo_part_1_3( _seconds, (x,y) ):
	demo.state = STATE_1_2

	y += 80
	text = TextSprite(	"Original by Rodney McAuley (1983)", \
					   20, (0,255,255), (x,y) )
	demo.sprite_set.add( text )
	y = text.rect.bottom+10
	
	text1 = TextSprite( "http://sizer99.com/wavy/", 24, (255,255,255), (x,y ) )
	y = text1.rect.bottom+10
	demo.sprite_set.add( text1 )
	

	# Just add the text, explosions have already killed themselves
	text2 = RainbowText( "Hit Spacebar to Play", \
					   24, ((255,255,255),(255,0,0),(0,0,255)), (x,y), 8 )
	demo.sprite_set.add( text2 )

	# Pause 10 seconds before going to rolling demo
	TimedEvent.add( 10, demo_part_2_1, _data=(text1,text2) )

def demo_part_2_1( _seconds, _save ):

	demo.state = STATE_2_1

	# Empty out the sprite set, then put back in the http and spacebar
	demo.sprite_set.empty()

	for sprite in _save:
		demo.sprite_set.add( sprite )

	Game.level_init( Game.level_get_config( 4 ) )

	# And after 30 seconds, start all over again
	TimedEvent.add( 30, demo_part_1_1 )


# -----------------------------------------------------------------------------
# Class DemoCloud
#
# This is basically just an empty black space that we draw on with clouds
# -----------------------------------------------------------------------------
class DemoCloud( pygame.sprite.Sprite ):
	"""
	And empty black area that we draw on with clouds - it animates between
	two states of fluffiness. MUST call preload() first to get approrpriate
	cell size.
	"""

	# pygame.image[] of the two clouds
	smallclouds = []
	interval = 7		# Everything is a multiple of this

	#@staticmethod
	def preload():
		"""
		This must be called before you create any DemoCloud() objects.
		Returns the interval at which we will draw clouds - cell size
		should be a multiple of this.
		"""
		DemoCloud.smallclouds = Util.image_loader( ( 'smoke_s', 'smoke_l' ) )
		DemoCloud.interval = (DemoCloud.smallclouds[0].get_width()+1)/2
		return DemoCloud.interval
	# Pre python 2.4 syntax to make it static - sigh
	preload = staticmethod( preload )

	def __init__( self, _topleft, _cells, _cell_size ):
		"""
		_topleft = top left position on screen
		_cells = number of cells (x,y)
		_cell_size = pixels per cell
		"""

		# Always init the sprite
		pygame.sprite.Sprite.__init__( self )

		# clamp to the DemoCloud.interval
		self.pos = ( _topleft[0] - (_topleft[0]%DemoCloud.interval),
					 _topleft[1] - (_topleft[1]%DemoCloud.interval) )
		self.cells = _cells
		self.cell_size = _cell_size
		self.size = ( (_cells[0]+2) * _cell_size, (_cells[1]+2) * _cell_size )

		# Create two frames, make them black
		frames = [ pygame.Surface( self.size ).convert() for dummy in DemoCloud.smallclouds ]
		for frame in frames:
			frame.fill( (0,0,0) )
			frame.set_colorkey( (0,0,0) )

		# create an imageset of the two frames, animate slowly
		self.imageset = ImageSet( "cloud", frames, _center=None, _interval=0.5 )
		self.update(0)

	def draw_smoke( self, _pos ):
		"""
		_pos is the plane position in the cloud - draw on both cloud images with
		the small and large cloud images appropriately, if the _pos is right.
		"""

		spacing = DemoCloud.interval
		# If we're on a grid boundary, draw a cloud
		if (_pos[0]%spacing)==0 and (_pos[1]%spacing)==0:
			clouds = DemoCloud.smallclouds
			# Alternate for each row, so it looks slightly less regular
			if _pos[1]%2:
				clouds.reverse()
			# Now draw each frame on each layer
			for i in range(len( DemoCloud.smallclouds)):
				self.imageset.frames[i].blit(
					clouds[i],
					(_pos[0]-DemoCloud.interval, _pos[1]-DemoCloud.interval ) )

	def update( self, _seconds ):
		"""
		Not much to do here, just switch between clouds every so often
		"""
		self.image, self.rect = self.imageset.newframe( _seconds, self.pos )
		self.rect.topleft = self.pos


# -----------------------------------------------------------------------------
# Class DemoFighter
#
# Use the Fighter() sprite to zoom around and draw our clouds
# -----------------------------------------------------------------------------

class DemoFighter( Fighter ):


	def __init__( self, _id, _cloud, _moves ):
		"""
		_id = For debugging
		_cloud = DemoCloud() we're flying on
		_moves = move set to follow - first move must go to (0,0)
		"""

		self.cloud = _cloud
		self.moves = _moves
		self.interval = _cloud.interval
		self.cell_size = _cloud.cell_size
		self.done = False

		# start off screen, moving down
		self.dir = (0, 1)
		self.move_idx = 0
		self.draw = False

		self.x, self.y = ( self.cell_size/2, -(self.cell_size*7)/2 )
		self.new_move( False )

		# init a fighter = some dummy settings
		Plane.base_newdir = self.dir
		Fighter.__init__( self, _id, _offset=(0,0), _callback=None )
		self.set_imageset( self.dir )

		# get the real settings


	def new_move( self, _showdir=True ):
		"""
		Move onto the next move in the series. Returns True if there
		are more, otherwise False
		"""

		# next destination, then convert from cell coords to real coords
		move = self.moves[self.move_idx]
		self.dir = move[0]
		
		self.destx = self.x + self.dir[0]*move[1]*self.cell_size
		self.desty = self.y + self.dir[1]*move[1]*self.cell_size
		try:
			self.draw = move[2]
		except IndexError:
			self.draw = False
			
		#print "x/y:", self.x, self.y, "destxy:", self.destx, self.desty, \
		#	  "dir:", self.dir, "draw:", self.draw

		if self.draw:
			self.cloud.draw_smoke( (self.x,self.y) )

		if _showdir:
			self.set_imageset( self.dir )

		self.move_idx += 1
		if self.move_idx >= len( self.moves ):
			self.move_idx -= 1
			return False
		else:
			return True

	def update( self, _seconds ):
		#print self.x, self.y, " - ", self.destx, self.desty

		donex = ( (self.dir[0]>0 and self.x>=self.destx) or 
				  (self.dir[0]<0 and self.x<=self.destx) or
				  not self.dir[0] )
		doney = ( (self.dir[1]>0 and self.y>=self.desty) or
				  (self.dir[1]<0 and self.y<=self.desty) or
				  not self.dir[1] )
		if donex and doney:	
			self.x, self.y = self.destx, self.desty
			if not self.new_move():
				# Mark as done, but keep going!
				self.done = True
		
		# figure out new position - we do this all in int steps so we
		# can easily draw the clouds at grid points
		speed = int(round( Plane.patrol_speed * _seconds ))
		speed = max( 1, speed )
		#speed *= 4     # fast for debug

		# For each pixels of speed, add the direction and then draw the
		# cloud if we need to
		for i in range( speed ):
			self.x += self.dir[0]
			self.y += self.dir[1]
			if self.draw:
				self.cloud.draw_smoke( (self.x,self.y) )

		# and make it relative to the cloud
		self.pos = ( self.x+self.cloud.rect.left, self.y+self.cloud.rect.top )
		self.image, self.rect = self.imageset.newframe( _seconds, self.pos )

	

# -----------------------------------------------------------------------------
#  The cloud/formation flying data
# -----------------------------------------------------------------------------


N  = ( 0,-1)
NE = ( 1,-1)
E  = ( 1, 0)
SE = ( 1, 1)
S  = ( 0, 1)
SW = (-1, 1)
W  = (-1, 0)
NW = (-1,-1)

#	"01234567890123456"
#0	"*   *  *  * * * *"
#1	"*   * * * * * * *"
#2	"*   * *** * *  * "
#3	"* * * * * * *  * "
#4	" * *  * *  *   * "

wavy_moves  = (
	# 'W'
	(  S, 4 ),		# S to top of W
	# Draw left of W		
	(  S, 4, True ),
	# Draw left \ 
	( SE, 1, True),
	# Loop down and back
	( SE, 2 ),		(  S, 1 ),		( SW, 1 ),		(  W, 2 ),
	( NW, 1 ),		(  N, 1 ),		( NE, 2 ),
	# Draw left /
	( NE, 1, True ),
	# Loop up and left
	( NE, 3 ),		(  N, 1 ),		( NW, 1 ),		(  W, 2 ),
	( SW, 1 ),		(  S, 3 ), 		( SE, 1 ),
	# Draw right \
	( SE, 1, True ),
	# Loop down and left
	( SE, 2),		(  S, 1 ),		( SW, 1 ),		(  W, 2 ),
	( NW, 1),		(  N, 1 ),		( NE, 2 ),
	# Draw right /
	( NE, 1, True ),
	# Draw the right of the W
	(  N, 4, True ),
	# Go up to get ready for A
	( NE, 1 ),		(  E, 1 ),

	# 'A'
	# Down to peak
	( SE, 1 ),
	# Draw right \
	( SE, 1, True ),
	# Draw right of A
	(  S, 4, True ),
	# Loop down and left
	(  S, 2),		( SW, 1 ),		( NW, 1 ),		(  N, 2 ),
	# Draw the left of A
	(  N, 4, True ),
	# Draw left /
	( NE, 1, True ),
	# Loop right and down
	( NE, 1 ),		(  E, 1 ),		( SE, 1 ),		( S, 2 ),
	( SW, 1 ), 		(  W, 1 ),
	# Draw - of A
	(  W, 2, True ),
		
	# Back up and and right
	( NW, 1 ), 		(  N, 2 ),		( NE, 1 ),		( E, 3 ),

	# 'V'
	( SE, 1 ),
	# Draw left |
	(  S, 4, True ),
	# Draw left \ 
	( SE, 1, True ),
	# Loop down and back to left
	( SE, 3 ),		(  S, 1 ),		( SW, 1),		( W, 1 ),
	( NW, 1 ),		(  N, 4 ),
	# Draw right /
	( NE, 1, True ),
	# Draw right |
	(  N, 4, True ),

	(  N, 1 ),		(  NE, 1 ),		(  E, 1 ), 		( SE, 2 ),

	# 'Y'
	# Draw right side of Y
	(  S, 2, True ), ( SW, 1, True ),
	# Loop down and back
	( SW, 3 ),		(  S, 1 ),		( SE, 1),		( E, 1 ),
	( NE, 1 ),		(  N, 4 ), 
	# Draw left side of Y
	( NW, 1, True ), (  N, 2, True ),
	# Loop up a bit then right then back down
	( NW, 2 ),		(  N, 1 ),		( NE, 1 ),		( E, 1 ),
	( SE, 1 ),		(  S, 6 ),
	# Draw bottom of 'Y'
	(  S, 2, True ),

	# Loop down and left, come back headed right for collision
	(  S, 5 ), 		( SW, 1 ),		(  W, 6 ),		( NW, 1 ),
	(  N, 1 ),		( NE, 1 ),		(  E, 1 )
		
)


#	"*   *  *  * * * *"
#	"**  * * * * * * *"
#	"* * * *** * *  * "
#	"*  ** * * * *  * "
#	"*   * * *  *   * "

navy_moves  = (
	# 'N'
	(  S, 4 ),		# S to top of N
	# Draw left of N
	(  S, 5, True ),
	# Loop down and back
	(  S, 3 ),		( SE, 1 ),		(  E, 1 ),		( NE, 1 ),		(  N, 3 ),
	# Draw right of N
	(  N, 5, True ),
	# Loop waaaay left
	(  N, 3 ), 		( NW, 1 ),		(  W, 1 ),		( SW, 1 ),		(  S, 4 ),
	# Draw \ of N
	( SE, 3, True ),

	# Head over for the A
	( SE, 2 ), 		( E, 1 ), 		( NE, 1 ),

	# 'A'
	# Do the side and top right side of the A
	(  N, 4, True ), ( NW, 1, True ),
	# Loop up and back for the left side
	( NW, 3 ),		(  N, 2 ),		( NE, 2 ),		(  E, 1 ),
	( SE, 2 ),		(  S, 3 ),		( SW, 2 ),
	# Draw the left side / and |
	( SW, 1, True ), (  S, 4, True ),
	# Loop back up left to get the -
	( SW, 2 ),		(  W, 2 ),		( NW, 1 ),		(  N, 1 ),
	( NE, 2 ),		(  E, 3 ),
	# - of the A
	(  E, 2, True ),

	# Head over for V
	(  E, 5 ),		( SE, 1 ),		( SW, 1 ),		(  W, 2 ),

	# 'V'
	# \ and left | of V
	( NW, 1, True ), (N, 4, True ),
	# Loop up and back to right side
	( NW, 2 ),		(  N, 2 ), 		( NE, 1 ),		(  E, 1 ),
	( SE, 2 ),		(  S, 3 ),
	# right | and / of V
	(  S, 4, True ), ( SW, 1, True ),

	# Loop down right to get bottom of Y
	(  S, 2 ),		( SE, 1 ), 		(  E, 1 ),		( NE, 2 ),		(  N, 1 ),

	# 'Y'
	# draw bottom, then left side
	(  N, 2, True ), (NW, True, 1 ), ( N, 2, True ),
	# Loop back up around to right to get right side
	( NW, 2 ),		( NE, 1 ),		(  E, 1 ),
	( SE, 2 ),		(  S, 1 ),
	# Draw right side
	(  S, 2, True ), ( SW, 1, True ),

	# Head out bottom
	( S, 2 ),

	# Loop down and right, come back headed left for collision
	(  S, 5 ), 		( SE, 1 ),		(  E, 2 ),		( NE, 1 ),
	(  N, 1 ),		( NW, 1 ),		(  W, 1 )
) 



# High speed, for debugging
#wavy_moves = ( ( S, 10 ),	( E, 1 ) )
#navy_moves = (	( S, 10 ),	( W, 1 ) )
	
